@@ -23,6 +23,82 @@ class GeoDataType:
     SeriesMatrix = 1
 
 
+class AccessionDownloader:
+
+    def __init__(self, accession_save_filepath: str = None, history_dir: str = None):
+        self.session = Session()
+        headers = {
+            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.114 Safari/537.36 Edg/89.0.774.76'}
+        self.session.headers = headers
+        self.history_dir = history_dir if history_dir else 'history'
+        self.success_filepath = f'{self.history_dir}/success_accession_page.txt'
+        self.accession_list = []
+        self.accession_save_filepath = accession_save_filepath if accession_save_filepath else 'accession_list.txt'
+
+        if not os.path.exists(self.history_dir):
+            os.mkdir(self.history_dir)
+
+    @Timeit(prefix='成功读取并完成过滤')
+    def filter_request(self, display: int, page_nums: int):
+        filter_string = f'每页 {display} 个, 共 {page_nums} 页'
+
+        if os.path.exists(self.success_filepath):
+            self.success_f = open(self.success_filepath, mode='a+')
+            self.success_f.seek(0)
+            success_pages = {int(line.strip()) for line in self.success_f}
+            page_list = {page for page in range(1, page_nums + 1) if page not in success_pages}
+            filter_string += f', 成功下载：{len(success_pages)}页'
+        else:
+            self.success_f = open(self.success_filepath, mode='a')
+            page_list = set(range(1, page_nums + 1))
+
+        if os.path.exists(self.accession_save_filepath):
+            self.accession_save_f = open(self.accession_save_filepath, mode='a+')
+            self.accession_save_f.seek(0)
+            success_count = len({row.strip() for row in self.accession_save_f})
+            filter_string += f' -> {success_count} 个'
+        else:
+            self.accession_save_f = open(self.accession_save_filepath, mode='w')
+
+        if page_list:
+            filter_string += f', 本次需下载 {len(page_list)} 页'
+        else:
+            filter_string += f', 全部下载成功'
+        print(filter_string)
+        return page_list
+
+    def spider(self, display, page):
+        url = f'https://www.ncbi.nlm.nih.gov/geo/browse/?view=series&zsort=date&display={display}&page={page}'
+        print(f'make request to {url}')
+        return page, self.session.get(url)
+
+    def parse(self, future):
+        try:
+            page, response = future.result()
+            html = etree.HTML(response.content)
+            accessions = html.xpath('//table[@id="geo_data"]/tbody/tr/td[1]/a/text()')
+            for accession in accessions:
+                self.accession_save_f.write(f'{accession}\n')
+            self.success_f.write(f'{page}\n')
+            self.accession_save_f.flush()
+            self.success_f.flush()
+            print(page, len(accessions))
+        except Exception as e:
+            print(e)
+
+    @Timeit(prefix='Accessions 下载完毕')
+    def start(self, workers: int = None, page_nums: int = 299, display: int = 500):
+        workers = workers if workers else os.cpu_count()
+        while True:
+            page_list = self.filter_request(display=display, page_nums=page_nums)
+            if not page_list:
+                break
+            with ThreadPoolExecutor(max_workers=workers) as pool:
+                for page in page_list:
+                    task = pool.submit(self.spider, display, page)
+                    task.add_done_callback(self.parse)
+
+
 class GeoDownloader:
     def __init__(self, accession_list: Iterable = None, download_dir: str = None, history_dir: str = None,
                  success_filepath: str = None, error_filepath: str = None):
